@@ -1,99 +1,29 @@
 import subprocess
 import os
-import struct
-from . import flatten_list, _monitor_brand_lookup, filter_monitors, __cache__
-from typing import List, Tuple, Union, Optional
+from . import flatten_list, _monitor_brand_lookup, filter_monitors, __cache__, EDID
+from typing import List, Union, Optional
+import warnings
+
+# edid class used to be defined here as _EDID
+_EDID = EDID
 
 
-class _EDID:
-    '''
-    Simple structure and method to extract monitor serial and name from an EDID string.
+def warn_deprecated(name, alternative=None):
+    '''Temporary function to warn you that things have been deprecated
+    and will be removed in the next update.'''
+    if alternative is None:
+        alternative = name
 
-    The EDID parsing was created with inspiration from the [pyedid library](https://github.com/jojonas/pyedid)
-    '''
-    EDID_FORMAT = (
-        ">"     # big-endian
-        "8s"    # constant header (8 bytes)
-        "H"     # manufacturer id (2 bytes)
-        "H"     # product id (2 bytes)
-        "I"     # serial number (4 bytes)
-        "B"     # manufactoring week (1 byte)
-        "B"     # manufactoring year (1 byte)
-        "B"     # edid version (1 byte)
-        "B"     # edid revision (1 byte)
-        "B"     # video input type (1 byte)
-        "B"     # horizontal size in cm (1 byte)
-        "B"     # vertical size in cm (1 byte)
-        "B"     # display gamma (1 byte)
-        "B"     # supported features (1 byte)
-        "10s"   # color characteristics (10 bytes)
-        "H"     # supported timings (2 bytes)
-        "B"     # reserved timing (1 byte)
-        "16s"   # EDID supported timings (16 bytes)
-        "18s"   # detailed timing block 1 (18 bytes)
-        "18s"   # detailed timing block 2 (18 bytes)
-        "18s"   # detailed timing block 3 (18 bytes)
-        "18s"   # detailed timing block 4 (18 bytes)
-        "B"     # extension flag (1 byte)
-        "B"     # checksum (1 byte)
-    )
-
-    @classmethod
-    def parse_edid(cls, edid: str) -> Tuple[Union[str, None], str]:
-        '''
-        Takes an EDID string (as string hex, formatted as: '00ffffff00...') and
-        attempts to extract the monitor's name and serial number from it
-
-        Args:
-            edid (str): the edid string
-
-        Returns:
-            tuple: First item can be None or str.
-                Second item is always str
-
-        Example:
-            ```python
-            import screen_brightness_control as sbc
-
-            edid = sbx.linux.list_monitors_info()[0]['edid']
-            name, serial = sbc.linux._EDID.parse_edid(edid)
-            if name is not None:
-                print('Success!')
-                print('Name:', name)
-                print('Serial:', serial)
-            else:
-                print('Unable to extract the data')
-            ```
-        '''
-        def filter_hex(st):
-            st = str(st)
-            while '\\x' in st:
-                i = st.index('\\x')
-                st = st.replace(st[i:i + 4], '')
-            return st.replace('\\n', '')[2:-1]
-
-        if ' ' in edid:
-            edid = edid.replace(' ', '')
-        edid = bytes.fromhex(edid)
-        data = struct.unpack(cls.EDID_FORMAT, edid)
-        serial = filter_hex(data[18])
-        # other info can be anywhere in this range, I don't know why
-        name = None
-        for i in data[19:22]:
-            try:
-                st = str(i)[2:-1].rstrip(' ').rstrip('\t')
-                if st.index(' ') < len(st) - 1:
-                    name = filter_hex(i).split(' ')
-                    name = name[0].lower().capitalize() + ' ' + name[1]
-            except Exception:
-                pass
-        return name, serial
+    msg = f'screen_brightness_control.linux.{name} is deprecated.'
+    if alternative:
+        msg += f' Use screen_brightness_control.{alternative} instead'
+    warnings.warn(msg, DeprecationWarning)
 
 
 class Light:
     '''collection of screen brightness related methods using the light executable'''
 
-    executable = 'light'
+    executable: str = 'light'
     '''the light executable to be called'''
 
     @classmethod
@@ -124,56 +54,52 @@ class Light:
             edp_info = sbc.linux.Light.get_display_info('edp-backlight')[0]
             ```
         '''
-        try:
-            displays = __cache__.get('light_monitors_info')
-        except Exception:
-            res = subprocess.run([cls.executable, '-L'], stdout=subprocess.PIPE).stdout.decode().split('\n')
-            displays = []
-            count = 0
-            for r in res:
-                if 'backlight' in r and 'sysfs/backlight/auto' not in r:
-                    r = r[r.index('backlight/') + 10:]
-                    if os.path.isfile(f'/sys/class/backlight/{r}/device/edid'):
-                        tmp = {
-                            'name': r,
-                            'path': f'/sys/class/backlight/{r}',
-                            'light_path': f'sysfs/backlight/{r}',
-                            'method': cls,
-                            'index': count,
-                            'model': None,
-                            'serial': None,
-                            'manufacturer': None,
-                            'manufacturer_id': None,
-                            'edid': None
-                        }
-                        count += 1
+        res = subprocess.run([cls.executable, '-L'], stdout=subprocess.PIPE).stdout.decode().split('\n')
+        displays = []
+        count = 0
+        for r in res:
+            if 'backlight' in r and 'sysfs/backlight/auto' not in r:
+                r = r[r.index('backlight/') + 10:]
+                if os.path.isfile(f'/sys/class/backlight/{r}/device/edid'):
+                    tmp = {
+                        'name': r,
+                        'path': f'/sys/class/backlight/{r}',
+                        'light_path': f'sysfs/backlight/{r}',
+                        'method': cls,
+                        'index': count,
+                        'model': None,
+                        'serial': None,
+                        'manufacturer': None,
+                        'manufacturer_id': None,
+                        'edid': None
+                    }
+                    count += 1
+                    try:
+                        out = subprocess.check_output(
+                            ['hexdump', tmp['path'] + '/device/edid'],
+                            stderr=subprocess.DEVNULL
+                        ).decode().split('\n')
+                        # either the hexdump reports each hex char backwards (ff00 instead of 00ff)
+                        # or both xrandr and ddcutil do. So I swap these bits around
+                        edid = ''
+                        for line in out:
+                            for i in line.split(' '):
+                                if len(i) == 4:
+                                    edid += i[2:] + i[:2]
+                        tmp['edid'] = edid
+                        name, serial = EDID.parse_edid(edid)
+                        if name is not None:
+                            tmp['serial'] = serial
+                            tmp['name'] = name
                         try:
-                            out = subprocess.check_output(
-                                ['hexdump', tmp['path'] + '/device/edid'],
-                                stderr=subprocess.DEVNULL
-                            ).decode().split('\n')
-                            # either the hexdump reports each hex char backwards (ff00 instead of 00ff)
-                            # or both xrandr and ddcutil do. So I swap these bits around
-                            edid = ''
-                            for line in out:
-                                for i in line.split(' '):
-                                    if len(i) == 4:
-                                        edid += i[2:] + i[:2]
-                            tmp['edid'] = edid
-                            name, serial = _EDID.parse_edid(edid)
-                            if name is not None:
-                                tmp['serial'] = serial
-                                tmp['name'] = name
-                            try:
-                                tmp['manufacturer_id'], tmp['manufacturer'] = _monitor_brand_lookup(name.split(' ')[0])
-                            except Exception:
-                                tmp['manufacturer'] = name.split(' ')[0]
-                                tmp['manufacturer_id'] = None
-                            tmp['model'] = name.split(' ')[1]
+                            tmp['manufacturer_id'], tmp['manufacturer'] = _monitor_brand_lookup(name.split(' ')[0])
                         except Exception:
-                            pass
-                        displays.append(tmp)
-            __cache__.store('light_monitors_info', displays)
+                            tmp['manufacturer'] = name.split(' ')[0]
+                            tmp['manufacturer_id'] = None
+                        tmp['model'] = name.split(' ')[1]
+                    except Exception:
+                        pass
+                    displays.append(tmp)
 
         if display is not None:
             displays = filter_monitors(display=display, haystack=displays, include=['path', 'light_path'])
@@ -182,6 +108,7 @@ class Light:
     @classmethod
     def get_display_names(cls) -> List[str]:
         '''
+        **DEPRECATED**.
         Returns the names of each display, as reported by light
 
         Returns:
@@ -195,28 +122,17 @@ class Light:
             # EG output: ['edp-backlight']
             ```
         '''
+        warn_deprecated('Light.get_display_names', 'list_monitors(method="light")')
         return [i['name'] for i in cls.get_display_info()]
 
     @classmethod
-    def set_brightness(
-        cls, value: int,
-        display: Optional[Union[int, str]] = None,
-        no_return: bool = False
-    ) -> Union[List[int], None]:
+    def set_brightness(cls, value: int, display: Optional[int] = None):
         '''
         Sets the brightness for a display using the light executable
 
         Args:
             value (int): Sets the brightness to this value
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial, path or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
-            no_return (bool): if True, this function returns None
-
-        Returns:
-            list: list of ints (0 to 100) (the result of `Light.get_brightness`)
-            None: if the `no_return` kwarg is True
+            display (int): The specific display you wish to query.
 
         Example:
             ```python
@@ -228,30 +144,24 @@ class Light:
             # set the primary display brightness to 75%
             sbc.linux.Light.set_brightness(75, display = 0)
 
-            # set the display called 'edp-backlight' to 25%
-            sbc.linux.Light.set_brightness(75, display = 'edp-backlight')
+            # set the secondary display brightness to 25%
+            sbc.linux.Light.set_brightness(75, display = 1)
             ```
         '''
         info = cls.get_display_info()
         if display is not None:
-            if type(display) == int:
-                info = [info[display]]
-            else:
-                info = filter_monitors(display=display, haystack=info, include=['path', 'light_path'])
+            info = [info[display]]
+
         for i in info:
             subprocess.call(f'{cls.executable} -S {value} -s {i["light_path"]}'.split(" "))
-        return cls.get_brightness(display=display) if not no_return else None
 
     @classmethod
-    def get_brightness(cls, display: Optional[Union[int, str]] = None) -> List[int]:
+    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
         '''
         Sets the brightness for a display using the light executable
 
         Args:
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial, path or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
+            display (int): The specific display you wish to query.
 
         Returns:
             list: list of ints (0 to 100)
@@ -266,16 +176,14 @@ class Light:
             # get the brightness of the primary display
             primary_brightness = sbc.linux.Light.get_brightness(display = 0)[0]
 
-            # get the brightness of the display called 'edp-backlight'
-            edp_brightness = sbc.linux.Light.get_brightness(display = 'edp-backlight')[0]
+            # get the brightness of the secondary display
+            edp_brightness = sbc.linux.Light.get_brightness(display = 1)[0]
             ```
         '''
         info = cls.get_display_info()
         if display is not None:
-            if type(display) == int:
-                info = [info[display]]
-            else:
-                info = filter_monitors(display=display, haystack=info, include=['path', 'light_path'])
+            info = [info[display]]
+
         results = []
         for i in info:
             results.append(
@@ -292,21 +200,17 @@ class Light:
 class XBacklight:
     '''collection of screen brightness related methods using the xbacklight executable'''
 
-    executable = 'xbacklight'
+    executable: str = 'xbacklight'
     '''the xbacklight executable to be called'''
 
     @classmethod
-    def set_brightness(cls, value: int, no_return: bool = False, **kwargs) -> Union[int, None]:
+    def set_brightness(cls, value: int, **kwargs):
         '''
         Sets the screen brightness to a supplied value
 
         Args:
-            no_return (bool): if True, this function returns None
-                Returns the result of `XBacklight.get_brightness()` otherwise
-
-        Returns:
-            int: from 0 to 100
-            None: if `no_return` is set to `True`
+            value (int): the value to set the brightnes to
+            **kwargs: arbitrary keyword arguments. Ignored
 
         Example:
             ```python
@@ -317,7 +221,6 @@ class XBacklight:
             ```
         '''
         subprocess.call([cls.executable, '-set', str(value)])
-        return cls.get_brightness() if not no_return else None
 
     @classmethod
     def get_brightness(cls, **kwargs) -> int:
@@ -326,6 +229,7 @@ class XBacklight:
 
         Returns:
             int: from 0 to 100
+            **kwargs: arbitrary keyword arguments. Ignored
 
         Example:
             ```python
@@ -344,11 +248,11 @@ class XBacklight:
 class XRandr:
     '''collection of screen brightness related methods using the xrandr executable'''
 
-    executable = 'xrandr'
+    executable: str = 'xrandr'
     '''the xrandr executable to be called'''
 
     @classmethod
-    def get_display_info(cls, display: Optional[Union[int, str]] = None) -> List[dict]:
+    def get_display_info(cls, display: Optional[Union[int, str]] = None, brightness: bool = False) -> List[dict]:
         '''
         Returns info about all detected monitors as reported by xrandr
 
@@ -356,6 +260,8 @@ class XRandr:
             display (str or int): [*Optional*] The monitor to return info about.
                 Pass in the serial number, name, model, interface, edid or index.
                 This is passed to `filter_monitors`
+            brightness (bool): whether to include the current brightness
+                in the returned info
 
         Returns:
             list: list of dicts
@@ -385,65 +291,61 @@ class XRandr:
                     return True
             return False
 
-        try:
-            data = __cache__.get('xrandr_monitors_info')
-        except Exception:
-            out = subprocess.check_output([cls.executable, '--verbose']).decode().split('\n')
-            names = cls.get_display_interfaces()
-            data = []
-            tmp = {}
-            count = 0
-            for i in out:
-                if i != '':
-                    if i.startswith(tuple(names)):
-                        if check_tmp(tmp):
-                            data.append(tmp)
-                        tmp = {
-                            'interface': i.split(' ')[0],
-                            'name': i.split(' ')[0],
-                            'line': i,
-                            'method': cls,
-                            'index': count,
-                            'model': None,
-                            'serial': None,
-                            'manufacturer': None,
-                            'manufacturer_id': None,
-                            'edid': None
-                        }
-                        count += 1
-                    elif 'EDID:' in i:
-                        st = out[out.index(tmp['line']):]
-                        edid = []
-                        for j in range(st.index(i) + 1, st.index(i) + 9):
-                            edid.append(st[j].replace('\t', '').replace(' ', ''))
-                        edid = ''.join(edid)
-                        tmp['edid'] = edid
-                        name, serial = _EDID.parse_edid(edid)
-                        tmp['name'] = name if name is not None else tmp['interface']
-                        if name is not None:
-                            tmp['manufacturer'] = name.split(' ')[0]
-                            try:
-                                tmp['manufacturer_id'], tmp['manufacturer'] = _monitor_brand_lookup(
-                                    tmp['manufacturer']
-                                )
-                            except Exception:
-                                tmp['manufacturer_id'] = None
-                            tmp['model'] = name.split(' ')[1]
-                            tmp['serial'] = serial
-                    elif 'Brightness:' in i:
-                        tmp['brightness'] = int(
-                            float(i.replace('Brightness:', '').replace(' ', '').replace('\t', '')) * 100
-                        )
-            if check_tmp(tmp):
-                data.append(tmp)
+        out = subprocess.check_output([cls.executable, '--verbose']).decode().split('\n')
+        names = cls.get_display_interfaces()
+        data = []
+        tmp = {}
+        count = 0
+        for i in out:
+            if i != '':
+                if i.startswith(tuple(names)):
+                    if check_tmp(tmp):
+                        data.append(tmp)
+                    tmp = {
+                        'interface': i.split(' ')[0],
+                        'name': i.split(' ')[0],
+                        'line': i,
+                        'method': cls,
+                        'index': count,
+                        'model': None,
+                        'serial': None,
+                        'manufacturer': None,
+                        'manufacturer_id': None,
+                        'edid': None
+                    }
+                    count += 1
+                elif 'EDID:' in i:
+                    st = out[out.index(tmp['line']):]
+                    edid = []
+                    for j in range(st.index(i) + 1, st.index(i) + 9):
+                        edid.append(st[j].replace('\t', '').replace(' ', ''))
+                    edid = ''.join(edid)
+                    tmp['edid'] = edid
+                    name, serial = EDID.parse_edid(edid)
+                    tmp['name'] = name if name is not None else tmp['interface']
+                    if name is not None:
+                        tmp['manufacturer'] = name.split(' ')[0]
+                        try:
+                            tmp['manufacturer_id'], tmp['manufacturer'] = _monitor_brand_lookup(
+                                tmp['manufacturer']
+                            )
+                        except Exception:
+                            tmp['manufacturer_id'] = None
+                        tmp['model'] = name.split(' ')[1]
+                        tmp['serial'] = serial
+                elif 'Brightness:' in i and brightness:
+                    tmp['brightness'] = int(
+                        float(i.replace('Brightness:', '').replace(' ', '').replace('\t', '')) * 100
+                    )
+        if check_tmp(tmp):
+            data.append(tmp)
 
-            __cache__.store('xrandr_monitors_info', data)
         if display is not None:
             data = filter_monitors(display=display, haystack=data, include=['interface'])
         return data
 
-    @staticmethod
-    def get_display_interfaces() -> List[str]:
+    @classmethod
+    def get_display_interfaces(cls) -> List[str]:
         '''
         Returns the interfaces of each display, as reported by xrandr
 
@@ -454,16 +356,17 @@ class XRandr:
             ```python
             import screen_brightness_control as sbc
 
-            names = sbc.linux.XRandr.get_display_names()
+            names = sbc.linux.XRandr.get_display_interfaces()
             # EG output: ['eDP-1', 'HDMI1', 'HDMI2']
             ```
         '''
-        out = subprocess.check_output(['xrandr', '-q']).decode().split('\n')
+        out = subprocess.check_output([cls.executable, '-q']).decode().split('\n')
         return [i.split(' ')[0] for i in out if 'connected' in i and 'disconnected' not in i]
 
     @classmethod
     def get_display_names(cls) -> List[str]:
         '''
+        **DEPRECATED**.
         Returns the names of each display, as reported by xrandr
 
         Returns:
@@ -477,18 +380,16 @@ class XRandr:
             # EG output: ['BenQ GL2450HM', 'Dell U2211H']
             ```
         '''
+        warn_deprecated('XRandr.get_display_names', 'list_monitors(method="xrandr")')
         return [i['name'] for i in cls.get_display_info()]
 
     @classmethod
-    def get_brightness(cls, display: Optional[Union[int, str]] = None) -> List[int]:
+    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
         '''
         Returns the brightness for a display using the xrandr executable
 
         Args:
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial, interface or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
+            display (int): The specific display you wish to query.
 
         Returns:
             list: list of integers (from 0 to 100)
@@ -504,37 +405,21 @@ class XRandr:
             primary_brightness = sbc.linux.XRandr.get_brightness(display=0)[0]
             ```
         '''
-        monitors = cls.get_display_info()
+        monitors = cls.get_display_info(brightness=True)
         if display is not None:
-            if type(display) == int:
-                monitors = [monitors[display]]
-            else:
-                monitors = filter_monitors(display=display, haystack=monitors, include=['interface'])
+            monitors = [monitors[display]]
         brightness = [i['brightness'] for i in monitors]
 
         return brightness
 
     @classmethod
-    def set_brightness(
-        cls, value: int,
-        display: Optional[Union[int, str]] = None,
-        no_return: bool = False
-    ) -> Union[List[int], None]:
+    def set_brightness(cls, value: int, display: Optional[int] = None):
         '''
         Sets the brightness for a display using the xrandr executable
 
         Args:
             value (int): Sets the brightness to this value
             display (int): The specific display you wish to query.
-                Can be index, name, model, serial, interface or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
-            no_return (bool): if True, this function returns None
-                Returns the result of `XRandr.get_brightness()` otherwise
-
-        Returns:
-            list: list of ints (0 to 100) (the result of `XRandr.get_brightness`)
-            None: if the `no_return` kwarg is True
 
         Example:
             ```python
@@ -550,14 +435,7 @@ class XRandr:
         value = str(float(value) / 100)
         info = cls.get_display_info()
         if display is not None:
-            if type(display) == int:
-                info = [info[display]]
-            else:
-                info = filter_monitors(
-                    display=display,
-                    haystack=info,
-                    include=['interface']
-                )
+            info = [info[display]]
 
         for i in info:
             subprocess.run([cls.executable, '--output', i['interface'], '--brightness', value])
@@ -565,16 +443,14 @@ class XRandr:
         # The get_brightness method takes the brightness value from get_display_info
         # The problem is that that display info is cached, meaning that the brightness
         # value is also cached. We must expire it here.
-        __cache__.expire('xrandr_monitors_info')
-        return cls.get_brightness(display=display) if not no_return else None
 
 
 class DDCUtil:
     '''collection of screen brightness related methods using the ddcutil executable'''
 
-    executable = 'ddcutil'
+    executable: str = 'ddcutil'
     '''the ddcutil executable to be called'''
-    sleep_multiplier = 0.5
+    sleep_multiplier: float = 0.5
     '''how long ddcutil should sleep between each DDC request (lower is shorter).
     See [the ddcutil docs](https://www.ddcutil.com/performance_options/) for more info.'''
 
@@ -616,9 +492,8 @@ class DDCUtil:
                 return True
             return False
 
-        try:
-            data = __cache__.get('ddcutil_monitors_info')
-        except Exception:
+        data = __cache__.get('ddcutil_monitors_info')
+        if data is None:
             out = []
             # Use -v to get EDID string but this means output cannot be decoded.
             # Or maybe it can. I don't know the encoding though, so let's assume it cannot be decoded.
@@ -696,6 +571,7 @@ class DDCUtil:
     @classmethod
     def get_display_names(cls) -> List[str]:
         '''
+        **DEPRECATED**.
         Returns the names of each display, as reported by ddcutil
 
         Returns:
@@ -709,18 +585,16 @@ class DDCUtil:
             # EG output: ['Dell U2211H', 'BenQ GL2450H']
             ```
         '''
+        warn_deprecated('DDCUtil.get_display_names', 'list_monitors(method="ddcutil")')
         return [i['name'] for i in cls.get_display_info()]
 
     @classmethod
-    def get_brightness(cls, display: Optional[Union[int, str]] = None) -> List[int]:
+    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
         '''
         Returns the brightness for a display using the ddcutil executable
 
         Args:
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial, i2c bus or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
+            display (int): The specific display you wish to query.
 
         Returns:
             list: list of ints (0 to 100)
@@ -738,17 +612,12 @@ class DDCUtil:
         '''
         monitors = cls.get_display_info()
         if display is not None:
-            if type(display) == int:
-                monitors = [monitors[display]]
-            else:
-                monitors = filter_monitors(display=display, haystack=monitors, include=['i2c_bus'])
+            monitors = [monitors[display]]
+
         res = []
         for m in monitors:
-            try:
-                out = __cache__.get('ddcutil_' + m['edid'] + '_brightness')
-                if out is None:
-                    raise Exception
-            except Exception:
+            out = __cache__.get('ddcutil_' + m['edid'] + '_brightness')
+            if out is None:
                 out = subprocess.check_output(
                     [
                         cls.executable,
@@ -765,26 +634,13 @@ class DDCUtil:
         return res
 
     @classmethod
-    def set_brightness(
-        cls, value: int,
-        display: Optional[Union[int, str]] = None,
-        no_return: bool = False
-    ) -> Union[List[int], None]:
+    def set_brightness(cls, value: int, display: Optional[int] = None):
         '''
         Sets the brightness for a display using the ddcutil executable
 
         Args:
             value (int): Sets the brightness to this value
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial, i2c bus or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
-            no_return (bool): if True, this function returns None.
-                Returns the result of `DDCUtil.get_brightness()` otherwise
-
-        Returns:
-            list: list of ints (0 to 100)
-            None: if `no_return` is True
+            display (int): The specific display you wish to query.
 
         Example:
             ```python
@@ -799,27 +655,18 @@ class DDCUtil:
         '''
         monitors = cls.get_display_info()
         if display is not None:
-            if type(display) == int:
-                monitors = [monitors[display]]
-            else:
-                monitors = filter_monitors(display=display, haystack=monitors, include=['i2c_bus'])
+            monitors = [monitors[display]]
 
         __cache__.expire(startswith='ddcutil_', endswith='_brightness')
         for m in monitors:
             subprocess.run(
                 [
-                    cls.executable,
-                    'setvcp',
-                    '10',
-                    str(value),
-                    '-b',
-                    str(m['bus_number']),
+                    cls.executable, 'setvcp', '10', str(value),
+                    '-b', str(m['bus_number']),
                     f'--sleep-multiplier={cls.sleep_multiplier}'
                 ]
             )
 
-        return cls.get_brightness(display=display) if not no_return else None
-
 
 def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = False) -> List[dict]:
     '''
@@ -842,32 +689,24 @@ def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = Fa
         monitors = sbc.linux.list_monitors_info()
         for monitor in monitors:
             print('=======================')
-
             # the manufacturer name plus the model OR a generic name for the monitor, depending on the method
             print('Name:', monitor['name'])
-
             # the general model of the display
             print('Model:', monitor['model'])
-
-            # a unique string assigned by Windows to this display
+            # the serial of the display
             print('Serial:', monitor['serial'])
-
             # the name of the brand of the monitor
             print('Manufacturer:', monitor['manufacturer'])
-
             # the 3 letter code corresponding to the brand name, EG: BNQ -> BenQ
             print('Manufacturer ID:', monitor['manufacturer_id'])
-
             # the index of that display FOR THE SPECIFIC METHOD THE DISPLAY USES
             print('Index:', monitor['index'])
-
             # the method this monitor can be addressed by
             print('Method:', monitor['method'])
         ```
     '''
-    try:
-        return __cache__.get('linux_monitors_info', method=method, allow_duplicates=allow_duplicates)
-    except Exception:
+    info = __cache__.get('linux_monitors_info', method=method, allow_duplicates=allow_duplicates)
+    if info is None:
         methods = [XRandr, DDCUtil, Light]
         if method is not None:
             method = method.lower()
@@ -889,11 +728,12 @@ def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = Fa
                             edids.append(i['edid'])
                             info.append(i)
         __cache__.store('linux_monitors_info', info, method=method, allow_duplicates=allow_duplicates)
-        return info
+    return info
 
 
 def list_monitors(method: Optional[str] = None) -> List[str]:
     '''
+    **DEPRECATED**.
     Returns the names of all detected monitors
 
     Args:
@@ -910,12 +750,14 @@ def list_monitors(method: Optional[str] = None) -> List[str]:
         # EG output: ['BenQ GL2450HM', 'Dell U2211H', 'edp-backlight']
         ```
     '''
+    warn_deprecated('list_monitors')
     displays = [i['name'] for i in list_monitors_info(method=method)]
     return flatten_list(displays)
 
 
 def get_brightness_from_sysfiles(display: int = None) -> int:
     '''
+    **DEPRECATED**.
     Returns the current display brightness by reading files from `/sys/class/backlight`
 
     Args:
@@ -936,6 +778,7 @@ def get_brightness_from_sysfiles(display: int = None) -> int:
         # Eg Output: 100
         ```
     '''
+    warn_deprecated('get_brightness_from_sysfiles', False)
     backlight_dir = '/sys/class/backlight/'
     error = []
     # if function has not returned yet try reading the brightness file
@@ -971,9 +814,11 @@ def get_brightness_from_sysfiles(display: int = None) -> int:
 
 def __set_and_get_brightness(*args, display=None, method=None, meta_method='get', **kwargs) -> Union[List[int], None]:
     '''
+    **DEPRECATED**.
     Internal function, do not call. Either sets the brightness or gets it.
     Exists because set_brightness and get_brightness only have a couple differences
     '''
+    warn_deprecated('__set_and_get_brightness', '__brightness')
     errors = []
     try:  # filter known list of monitors according to kwargs
         if type(display) == int:
@@ -1028,6 +873,7 @@ def set_brightness(
     **kwargs
 ) -> Union[List[int], int, None]:
     '''
+    **DEPRECATED**.
     Sets the brightness for a display, cycles through Light, XRandr, DDCUtil and XBacklight methods until one works
 
     Args:
@@ -1063,6 +909,7 @@ def set_brightness(
         sbc.linux.set_brightness(25, method='xrandr')
         ```
     '''
+    warn_deprecated('set_brightness')
     if method is not None and method.lower() == 'xbacklight':
         return XBacklight.set_brightness(value, **kwargs)
     else:
@@ -1075,6 +922,7 @@ def get_brightness(
     **kwargs
 ) -> Union[List[int], int]:
     '''
+    **DEPRECATED**.
     Returns the brightness for a display, cycles through Light, XRandr, DDCUtil and XBacklight methods until one works
 
     Args:
@@ -1111,6 +959,7 @@ def get_brightness(
         light_brightness = sbc.get_brightness(display=1, method='light')[0]
         ```
     '''
+    warn_deprecated('get_brightness')
     if method is not None and method.lower() == 'xbacklight':
         return XBacklight.get_brightness(**kwargs)
     else:
