@@ -6,14 +6,27 @@ import time
 import ctypes
 from ctypes import windll, byref, Structure, WinError, POINTER, WINFUNCTYPE
 from ctypes.wintypes import BOOL, HMONITOR, HDC, RECT, LPARAM, DWORD, BYTE, WCHAR, HANDLE
-from . import flatten_list, _monitor_brand_lookup, filter_monitors, __cache__, platform
+from . import flatten_list, _monitor_brand_lookup, filter_monitors, __cache__, platform, EDID
 from typing import List, Union, Optional
+import warnings
 # a bunch of typing classes were deprecated in Python 3.9
 # in favour of collections.abc (https://www.python.org/dev/peps/pep-0585/)
 if int(platform.python_version_tuple()[1]) < 9:
-    from typing import Iterable
+    from typing import Generator
 else:
-    from collections.abc import Iterable
+    from collections.abc import Generator
+
+
+def warn_deprecated(name, alternative=None):
+    '''Temporary function to warn you that things have been deprecated
+    and will be removed in the next update.'''
+    if alternative is None:
+        alternative = name
+
+    msg = f'screen_brightness_control.windows.{name} is deprecated.'
+    if alternative:
+        msg += f' Use screen_brightness_control.{alternative} instead'
+    warnings.warn(msg, DeprecationWarning)
 
 
 def _wmi_init():
@@ -41,10 +54,8 @@ def get_display_info() -> List[dict]:
             print(display['name'])
         ```
     '''
-    try:
-        info = __cache__.get('windows_monitors_info_raw')
-    except Exception:
-        info = []
+    info = __cache__.get('windows_monitors_info_raw')
+    if info is None:
         try:
             # collect all monitor UIDs (derived from DeviceID)
             monitor_uids = {}
@@ -76,83 +87,62 @@ def get_display_info() -> List[dict]:
             # gather list of laptop displays to check against later
             wmi = _wmi_init()
             try:
+                laptop_displays = [
+                    i.InstanceName
+                    for i in wmi.WmiMonitorBrightness()
+                ]
+            except Exception:
                 laptop_displays = []
-                for i in wmi.WmiMonitorBrightness():
-                    laptop_displays.append(
-                        i.InstanceName.replace('_0', '', 1).split('\\')[2]
-                    )
-            except Exception:
-                pass
-            monitors = []
-            extras = []
+
+            extras, desktop, laptop = [], 0, 0
             uid_keys = list(monitor_uids.keys())
-            for m in wmi.WmiMonitorID():
-                name = m.InstanceName.replace('_0', '', 1).split('\\')[2]
-                if name in uid_keys:
-                    monitors.append(m)
-                else:
-                    extras.append(m)
-
-            # sort the monitors in the same order as win32api reports them
-            # because the first item in win32api's list is usually the primary display
-            monitors = sorted(
-                monitors,
-                key=lambda x: uid_keys.index(x.InstanceName.replace('_0', '', 1).split('\\')[2])
-            )
-
-            monitors += extras
-
-            # get all available edid strings
-            try:
-                descriptors = {
-                    i.InstanceName: i.WmiGetMonitorRawEEdidV1Block(0) for i in wmi.WmiMonitorDescriptorMethods()
-                }
-            except Exception:
-                pass
-            laptop = 0
-            desktop = 0
-            for monitor in monitors:
-                model, serial, manufacturer, man_id, edid = None, None, None, None, None
-                instance_name = monitor.InstanceName.replace('_0', '', 1).split('\\')[2]
-                pydevice = monitor_uids[instance_name]
-
+            for monitor in wmi.WmiMonitorDescriptorMethods():
                 try:
-                    serial = bytes(monitor.SerialNumberID).decode().replace('\x00', '')
-                    manufacturer, model = bytes(monitor.UserFriendlyName).decode().replace('\x00', '').split(' ')
-                    manufacturer = manufacturer.lower().capitalize()
+                    model, serial, manufacturer, man_id, edid = None, None, None, None, None
+                    instance_name = monitor.InstanceName.replace('_0', '', 1).split('\\')[2]
+                    pydevice = monitor_uids[instance_name]
+
+                    # get the EDID
                     try:
-                        man_id, manufacturer = _monitor_brand_lookup(manufacturer)
-                    except Exception:
-                        man_id = None
-                except Exception:
-                    devid = pydevice.DeviceID.split('#')
-                    serial = devid[2]
-                    man_id = devid[1][:3]
-                    model = devid[3:]
-                    del(devid)
-                    try:
-                        man_id, manufacturer = _monitor_brand_lookup(man_id)
-                    except Exception:
-                        manufacturer = None
-                try:
-                    try:
-                        chars = descriptors[monitor.InstanceName][0]
-                    except Exception:
-                        chars = descriptors[pydevice.InstanceName][0]
-                    finally:
                         edid = ''
-                        for char in chars:
-                            char = str(hex(char)).replace('0x', '')
+                        for char in monitor.WmiGetMonitorRawEEdidV1Block(0)[0]:
+                            char = hex(char)[2:]
                             if len(char) == 1:
                                 char = '0' + char
                             edid += char
-                        del(chars)
-                except Exception:
-                    edid = None
+                    except Exception:
+                        edid = None
 
-                if (serial, model) != (None, None):
-                    info.append(
-                        {
+                    # get serial, model, manufacturer and manufacturer ID
+                    try:
+                        if edid is None:
+                            raise Exception
+                        # we do the EDID parsing ourselves because calling wmi.WmiMonitorID
+                        # takes too long
+                        name, serial = EDID.parse_edid(edid)
+                        if name is None:
+                            raise Exception
+
+                        # split by last space because model numbers usually are one word
+                        # whereas brands can be multiple (EG: 'LG Electronics')
+                        manufacturer, model = name.rsplit(' ', 1)
+                        try:
+                            man_id, manufacturer = _monitor_brand_lookup(manufacturer)
+                        except Exception:
+                            man_id, manufacturer = None, manufacturer.lower().capitalize()
+                    except Exception:
+                        devid = pydevice.DeviceID.split('#')
+                        serial = devid[2]
+                        man_id = devid[1][:3]
+                        model = devid[3]
+                        del(devid)
+                        try:
+                            man_id, manufacturer = _monitor_brand_lookup(man_id)
+                        except Exception:
+                            manufacturer = None
+
+                    if (serial, model) != (None, None):
+                        data = {
                             'name': f'{manufacturer} {model}',
                             'model': model,
                             'serial': serial,
@@ -160,15 +150,32 @@ def get_display_info() -> List[dict]:
                             'manufacturer_id': man_id,
                             'edid': edid
                         }
-                    )
-                    if instance_name in laptop_displays:
-                        info[-1]['index'] = laptop
-                        info[-1]['method'] = WMI
-                        laptop += 1
-                    else:
-                        info[-1]['index'] = desktop
-                        info[-1]['method'] = VCP
-                        desktop += 1
+                        if monitor.InstanceName in laptop_displays:
+                            data['index'] = laptop
+                            data['method'] = WMI
+                            laptop += 1
+                        else:
+                            data['method'] = VCP
+                            desktop += 1
+
+                        if instance_name in uid_keys:
+                            # insert the data into the uid_keys list because
+                            # uid_keys has the monitors sorted correctly. This
+                            # means we don't have to re-sort the list later
+                            uid_keys[uid_keys.index(instance_name)] = data
+                        else:
+                            extras.append(data)
+                except Exception:
+                    pass
+
+            info = uid_keys + extras
+            if desktop:
+                # now make sure desktop monitors have the correct index
+                count = 0
+                for item in info:
+                    if item['method'] == VCP:
+                        item['index'] = count
+                        count += 1
         except Exception:
             pass
         __cache__.store('windows_monitors_info_raw', info)
@@ -210,9 +217,8 @@ class WMI:
             benq_info = sbc.windows.WMI.get_display_info('BenQ GL2450H')
             ```
         '''
-        try:
-            info = __cache__.get('wmi_monitor_info')
-        except Exception:
+        info = __cache__.get('wmi_monitor_info')
+        if info is None:
             info = [i for i in get_display_info() if i['method'] == cls]
             __cache__.store('wmi_monitor_info', info)
         if display is not None:
@@ -222,6 +228,7 @@ class WMI:
     @classmethod
     def get_display_names(cls) -> List[str]:
         '''
+        **DEPRECATED**.
         Returns names of all displays that can be addressed by WMI
 
         Returns:
@@ -235,29 +242,17 @@ class WMI:
                 print(name)
             ```
         '''
+        warn_deprecated('WMI.get_display_names', 'list_monitors(method="wmi")')
         return [i['name'] for i in cls.get_display_info()]
 
     @classmethod
-    def set_brightness(
-        cls, value: int,
-        display: Optional[Union[int, str]] = None,
-        no_return: bool = False
-    ) -> Union[List[int], None]:
+    def set_brightness(cls, value: int, display: Optional[int] = None):
         '''
         Sets the display brightness for Windows using WMI
 
         Args:
             value (int): The percentage to set the brightness to
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
-            no_return (bool): if True, this function returns None
-                Otherwise it returns the result of `WMI.get_brightness()`
-
-        Returns:
-            list: list of integers (0 to 100)
-            None: if `no_return` is set to `True`
+            display (int): The specific display you wish to query.
 
         Raises:
             LookupError: if the given display cannot be found
@@ -272,31 +267,24 @@ class WMI:
             # set the primary display brightness to 75%
             sbc.windows.WMI.set_brightness(75, display = 0)
 
-            # set the brightness of a named monitor to 25%
-            sbc.windows.WMI.set_brightness(25, display = 'BenQ GL2450H')
+            # set the brightness of the secondary display to 25%
+            sbc.windows.WMI.set_brightness(25, display = 1)
             ```
         '''
         brightness_method = _wmi_init().WmiMonitorBrightnessMethods()
         if display is not None:
-            if type(display) == int:
-                brightness_method = [brightness_method[display]]
-            else:
-                indexes = [i['index'] for i in filter_monitors(display=display, method='wmi')]
-                brightness_method = [brightness_method[i] for i in indexes]
+            brightness_method = [brightness_method[display]]
+
         for method in brightness_method:
             method.WmiSetBrightness(value, 0)
-        return cls.get_brightness(display=display) if not no_return else None
 
     @classmethod
-    def get_brightness(cls, display: Optional[Union[int, str]] = None) -> List[int]:
+    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
         '''
         Returns the current display brightness using WMI
 
         Args:
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
+            display (int): The specific display you wish to query.
 
         Returns:
             list: list of integers (0 to 100)
@@ -318,17 +306,13 @@ class WMI:
             # get the primary display brightness
             primary_brightness = sbc.windows.WMI.get_brightness(display = 0)
 
-            # get the brightness of a named monitor
-            benq_brightness = sbc.windows.WMI.get_brightness(display = 'BenQ GL2450H')
+            # get the brightness of the secondary monitor
+            benq_brightness = sbc.windows.WMI.get_brightness(display = 1)
             ```
         '''
         brightness_method = _wmi_init().WmiMonitorBrightness()
         if display is not None:
-            if type(display) == int:
-                brightness_method = [brightness_method[display]]
-            else:
-                displays = cls.get_display_info(display)
-                brightness_method = [brightness_method[i['index']] for i in displays]
+            brightness_method = [brightness_method[display]]
 
         values = [i.CurrentBrightness for i in brightness_method]
         return values
@@ -344,12 +328,15 @@ class VCP:
                     ('description', WCHAR * 128)]
 
     @classmethod
-    def iter_physical_monitors(cls) -> Iterable[ctypes.wintypes.HANDLE]:
+    def iter_physical_monitors(cls, start: int = 0) -> Generator[ctypes.wintypes.HANDLE, None, None]:
         '''
         A generator to iterate through all physical monitors
         and then close them again afterwards, yielding their handles.
         It is not recommended to use this function unless you are familiar with `ctypes` and `windll`
 
+        Args:
+            start (int): skip the first X handles
+
         Yields:
             ctypes.wintypes.HANDLE
 
@@ -371,19 +358,27 @@ class VCP:
         monitors = []
         if not windll.user32.EnumDisplayMonitors(None, None, cls._MONITORENUMPROC(callback), None):
             raise WinError('EnumDisplayMonitors failed')
+
+        index = 0
+
         for monitor in monitors:
             # Get physical monitor count
             count = DWORD()
             if not windll.dxva2.GetNumberOfPhysicalMonitorsFromHMONITOR(monitor, byref(count)):
                 raise WinError()
             if count.value > 0:
+                if start and index + count.value < start:
+                    index += count.value
+                    continue
                 # Get physical monitor handles
                 physical_array = (cls._PHYSICAL_MONITOR * count.value)()
                 if not windll.dxva2.GetPhysicalMonitorsFromHMONITOR(monitor, count.value, physical_array):
                     raise WinError()
                 for item in physical_array:
-                    yield item.handle
+                    if not (start and index != start):
+                        yield item.handle
                     windll.dxva2.DestroyPhysicalMonitor(item.handle)
+                    index += 1
 
     @classmethod
     def get_display_info(cls, display: Optional[Union[int, str]] = None) -> List[dict]:
@@ -411,9 +406,8 @@ class VCP:
             # EG output: {'name': 'BenQ GL2450H', 'model': 'GL2450H', ... }
             ```
         '''
-        try:
-            info = __cache__.get('vcp_monitor_info')
-        except Exception:
+        info = __cache__.get('vcp_monitor_info')
+        if info is None:
             info = [i for i in get_display_info() if i['method'] == cls]
             __cache__.store('vcp_monitor_info', info)
         if display is not None:
@@ -423,6 +417,7 @@ class VCP:
     @staticmethod
     def get_monitor_caps(monitor: ctypes.wintypes.HANDLE) -> str:
         '''
+        **DEPRECATED**.
         Fetches and returns the VCP capabilities string of a monitor.
         This function takes anywhere from 1-2 seconds to run
 
@@ -441,6 +436,7 @@ class VCP:
             # EG output: '(prot(monitor)type(LCD)model(GL2450HM)cmds(01 02 03 07 0C F3)vcp(02...)'
             ```
         '''
+        warn_deprecated('VCP.get_monitor_caps', False)
         caps_string_length = DWORD()
         if not windll.dxva2.GetCapabilitiesStringLength(monitor, ctypes.byref(caps_string_length)):
             return
@@ -452,6 +448,7 @@ class VCP:
     @classmethod
     def get_display_names(cls) -> List[str]:
         '''
+        **DEPRECATED**.
         Return the names of each detected monitor
 
         Returns:
@@ -466,18 +463,16 @@ class VCP:
             # EG output: ['BenQ GL2450H', 'Dell U2211H']
             ```
         '''
+        warn_deprecated('VCP.get_display_names', 'list_monitors(method="vcp")')
         return [i['name'] for i in cls.get_display_info()]
 
     @classmethod
-    def get_brightness(cls, display: Optional[Union[int, str]] = None) -> List[int]:
+    def get_brightness(cls, display: Optional[int] = None) -> List[int]:
         '''
         Retrieve the brightness of all connected displays using the `ctypes.windll` API
 
         Args:
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
+            display (int): The specific display you wish to query.
 
         Returns:
             list: list of ints (0 to 100)
@@ -495,72 +490,44 @@ class VCP:
 
             # Get the brightness for a secondary display
             secondary_brightness = sbc.windows.VCP.get_brightness(display = 1)[0]
-
-            # Get the brightness for a display with the model 'GL2450H'
-            benq_brightness = sbc.windows.VCP.get_brightness(display = 'GL2450H')[0]
             ```
         '''
-        # filter monitors even if display kwarg is not specified due to oddities surrounding issues #7 and #8
-        # https://github.com/Crozzers/screen_brightness_control/issues/7
-        # https://github.com/Crozzers/screen_brightness_control/issues/8
-        # essentially, we get 'ghost' monitors showing up here that cannot actually
-        # be adjusted (even if no error gets raised) so we use this to filter out
-        # such ghost monitors by only attempting to get the brightness for valid monitors
-        # (determined by VCP.get_display_info)
-        # yes, it does add an unnecessary function call but that's only if you're using this module low-level.
-        # Top-level functions always end up specifying the display kwarg anyway
-        if type(display) == int:
-            indexes = [display]
-        else:
-            indexes = [i['index'] for i in filter_monitors(display=display, haystack=cls.get_display_info())]
-
-        count = 0
+        code = BYTE(0x10)
         values = []
-        for m in cls.iter_physical_monitors():
-            try:
-                v = __cache__.get(f'vcp_brightness_{count}')
-            except Exception:
+        for index, monitor in enumerate(
+            cls.iter_physical_monitors(start=display),
+            start=display if display is not None else 0
+        ):
+            current = __cache__.get(f'vcp_brightness_{index}')
+            if current is None:
                 cur_out = DWORD()
+                handle = HANDLE(monitor)
                 for _ in range(10):
-                    if windll.dxva2.GetVCPFeatureAndVCPFeatureReply(HANDLE(m), BYTE(0x10), None, byref(cur_out), None):
-                        v = cur_out.value
+                    if windll.dxva2.GetVCPFeatureAndVCPFeatureReply(handle, code, None, byref(cur_out), None):
+                        current = cur_out.value
                         break
                     else:
+                        current = None
                         time.sleep(0.02)
-                        v = None
-                del(cur_out)
-            if v is not None:
-                if count in indexes:
-                    try:
-                        __cache__.store(f'vcp_brightness_{count}', v, expires=0.1)
-                    except IndexError:
-                        pass
-                    values.append(v)
-                    if count >= max(indexes):
-                        break
-            count += 1
+
+            if current is not None:
+                __cache__.store(f'vcp_brightness_{index}', current, expires=0.1)
+                values.append(current)
+
+            if display == index:
+                # if we have just got the display we wanted then exit here
+                # no point iterating through all the other ones
+                break
 
         return values
 
     @classmethod
-    def set_brightness(
-        cls, value: int,
-        display: Optional[Union[int, str]] = None,
-        no_return: bool = False
-    ) -> Union[List[int], None]:
+    def set_brightness(cls, value: int, display: Optional[int] = None):
         '''
         Sets the brightness for all connected displays using the `ctypes.windll` API
 
         Args:
-            display (int or str): The specific display you wish to query.
-                Can be index, name, model, serial or edid string.
-                `int` is faster as it isn't passed to `filter_monitors` to be matched against.
-                `str` is slower as it is passed to `filter_monitors` to match to a display.
-            no_return (bool): if set to `True` this function will return `None`
-
-        Returns:
-            list: list of ints (0 to 100) (the result of `VCP.get_brightness`) if `no_return` is False
-            None: if `no_return` is True
+            display (int): The specific display you wish to query.
 
         Examples:
             ```python
@@ -574,29 +541,22 @@ class VCP:
 
             # Set the brightness for a secondary display to 25%
             sbc.windows.VCP.set_brightness(25, display = 1)
-
-            # Set the brightness for a display with the model 'GL2450H' to 100%
-            sbc.windows.VCP.set_brightness(100, display = 'GL2450H')
             ```
         '''
-        if type(display) == int:
-            indexes = [display]
-        else:
-            # see VCP.set_brightness for the explanation for why we always gather this list
-            indexes = [i['index'] for i in filter_monitors(display=display, haystack=cls.get_display_info())]
-
         __cache__.expire(startswith='vcp_brightness_')
-
-        count = 0
-        for m in cls.iter_physical_monitors():
-            if display is None or (count in indexes):
+        code = BYTE(0x10)
+        for index, monitor in enumerate(
+            cls.iter_physical_monitors(start=display),
+            start=display if display is not None else 0
+        ):
+            if display is None or display == index:
+                handle = HANDLE(monitor)
+                value = DWORD(value)
                 for _ in range(10):
-                    if windll.dxva2.SetVCPFeature(HANDLE(m), BYTE(0x10), DWORD(value)):
+                    if windll.dxva2.SetVCPFeature(handle, code, value):
                         break
                     else:
                         time.sleep(0.02)
-            count += 1
-        return cls.get_brightness(display=display) if not no_return else None
 
 
 def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = False) -> List[dict]:
@@ -620,41 +580,32 @@ def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = Fa
         monitors = sbc.windows.list_monitors_info()
         for info in monitors:
             print('=======================')
-
             # the manufacturer name plus the model
             print('Name:', info['name'])
-
             # the general model of the display
             print('Model:', info['model'])
-
             # a unique string assigned by Windows to this display
             print('Serial:', info['serial'])
-
             # the name of the brand of the monitor
             print('Manufacturer:', info['manufacturer'])
-
             # the 3 letter code corresponding to the brand name, EG: BNQ -> BenQ
             print('Manufacturer ID:', info['manufacturer_id'])
-
             # the index of that display FOR THE SPECIFIC METHOD THE DISPLAY USES
             print('Index:', info['index'])
-
             # the method this monitor can be addressed by
             print('Method:', info['method'])
-
             # the EDID string of the monitor
             print('EDID:', info['edid'])
         ```
     '''
-    try:
-        return __cache__.get(f'windows_monitors_info_{method}_{allow_duplicates}')
-    except Exception:
+    info = __cache__.get(f'windows_monitors_info_{method}_{allow_duplicates}')
+    if info is None:
         info = get_display_info()
 
         if method is not None:
             method = method.lower()
             if method not in ('wmi', 'vcp'):
-                raise ValueError('method kwarg must be \'wmi\' or \'vcp\'')
+                raise ValueError('method kwarg must be "wmi" or "vcp"')
 
         info_final = []
         serials = []
@@ -665,11 +616,13 @@ def list_monitors_info(method: Optional[str] = None, allow_duplicates: bool = Fa
                     serials.append(i['serial'])
                     info_final.append(i)
         __cache__.store(f'windows_monitors_info_{method}_{allow_duplicates}', info_final)
-        return info_final
+        info = info_final
+    return info
 
 
 def list_monitors(method: Optional[str] = None) -> List[str]:
     '''
+    **DEPRECATED**.
     Returns a list of all addressable monitor names
 
     Args:
@@ -686,14 +639,17 @@ def list_monitors(method: Optional[str] = None) -> List[str]:
         # EG output: ['BenQ GL2450H', 'Dell U2211H']
         ```
     '''
+    warn_deprecated('list_monitors')
     return [i['name'] for i in list_monitors_info(method=method)]
 
 
 def __set_and_get_brightness(*args, display=None, method=None, meta_method='get', **kwargs) -> Union[List[int], None]:
     '''
+    **DEPRECATED**.
     Internal function, do not call. Either sets the brightness or gets it.
     Exists because set_brightness and get_brightness only have a couple differences
     '''
+    warn_deprecated('__set_and_get_brightness', '__brightness')
     errors = []
     try:  # filter known list of monitors according to kwargs
         if type(display) == int:
@@ -740,6 +696,7 @@ def set_brightness(
     **kwargs
 ) -> Union[List[int], None]:
     '''
+    **DEPRECATED**.
     Sets the brightness of any connected monitors
 
     Args:
@@ -776,6 +733,7 @@ def set_brightness(
         sbc.windows.set_brightness(100, display = 'BenQ GL2450H')
         ```
     '''
+    warn_deprecated('set_brightness')
     # this function is called because set_brightness and get_brightness only differed by 1 line of code
     # so I made another internal function to reduce the filesize
     return __set_and_get_brightness(value, display=display, method=method, meta_method='set', **kwargs)
@@ -783,6 +741,7 @@ def set_brightness(
 
 def get_brightness(display: Optional[Union[int, str]] = None, method: Optional[str] = None, **kwargs) -> List[int]:
     '''
+    **DEPRECATED**.
     Returns the brightness of any connected monitors
 
     Args:
@@ -821,6 +780,7 @@ def get_brightness(display: Optional[Union[int, str]] = None, method: Optional[s
         benq_brightness = sbc.windows.get_brightness(display='BenQ GL2450H')[0]
         ```
     '''
+    warn_deprecated('get_brightness')
     # this function is called because set_brightness and get_brightness only differed by 1 line of code
     # so I made another internal function to reduce the filesize
     return __set_and_get_brightness(display=display, method=method, meta_method='get', **kwargs)
